require 'gdbm'
require 'digest/md5'

#===============================================================================
#
#	スケジュール DB クラス
#
class ScheduleDB < GDBM

	def get_md5(key)
		it = self[key] || ','
		Digest::MD5.hexdigest(it)
	end

	# <item1>:<item2=value2>:<item3>,<event1>\n<event2>\n<event3>

	def get_items(key)
		it = self[key] || ','
		items = {}; it.split(',', 2)[0].split(':').each {|m|
			m =~ /(.+)=(.+)/ ? items[$1] = $2 : items[m] = true
		}
		items
	end

	def get_events(key)
		it = self[key] || ','
		it.split(',', 2)[1]
	end

	def get_mark(key)
		self[key]
	end

	def set_mark(key, value)
		self[key] = value
	end
end

#================================================================================
#
#	スケジューラクラス
#
class Scheduler

	begin
		$LOAD_PATH.unshift('/usr/local/lib/ruby')
		require 'my_holidays'
	rescue LoadError
		raise
	end

	attr_reader :script
	attr_reader :user
	attr_reader :post_value

	def initialize
		@now_time = Time.now
		@db_name = 'data/sche-%Y-%m.gdbm'
		@script = File.basename(ENV['SCRIPT_NAME'] || '')
		@user = @script.gsub(/\..*/, '')
		reset_month
	end

	def reset_month
		@cur_month = Time.local(@now_time.year, @now_time.month) - 1	# 先月にする
		forward_month
	end

	def forward_month(d = 1)
		last_month = @cur_month.month
		@cur_month += 28 * 24 * 60 * 60 * d while(@cur_month.month == last_month)
		@cur_time = Time.local(@cur_month.year, @cur_month.month)	# 今月の 1 日を求める
		@cur_time -= 24 * 60 * 60 while(@cur_time.wday != 0)	# 日曜まで戻る
	end

	def back_month
		forward_month(-1)
	end

	def cur_year
		@cur_month.year
	end

	def cur_month
		@cur_month.month
	end

	def set_week_colors(colors)
		@week_colors = colors
	end

	def set_week_names(names)
		@week_names = names
	end

	def week_color(wday)
		@week_colors[wday]
	end

	def week_name(wday)
		@week_names[wday]
	end

	def set_post_value(value)
		@post_value = value
	end

	def each_day
		db = ScheduleDB.new(@cur_month.strftime(@db_name))
		(0..50).each {
			day = ''; items = {}; events = '　'
			if(@cur_time.month == @cur_month.month)
				day = @cur_time.day.to_s
				items = db.get_items(day)
				(@now_time.yday == @cur_time.yday + 1) and events += "きのう\n"
				(@now_time.yday == @cur_time.yday    ) and events += "きょう\n"
				(@now_time.yday == @cur_time.yday - 1) and events += "あした\n"
				(it = @cur_time.my_holiday?) and events += "#{it}\n"
				events += db.get_events(day)
			end
			yield({	:TIME		=> @cur_time,
					:DAY		=> day,
					:TODAY		=> @cur_time.yday == @now_time.yday,
					:HOLIDAY	=> @cur_time.my_holiday?,
					:ITEMS		=> items,
					:EVENTS		=> events,
			})
			@cur_time += 24 * 60 * 60
			@cur_time.wday == 0 and @cur_time.month != @cur_month.month and break
		}
		db.close
	end

	def get_mark(target_time)
		db = ScheduleDB.new(target_time.strftime(@db_name))
		target_day = target_time.day.to_s
		it = { :MD5 => db.get_md5(target_day), :ITEMS => db.get_items(target_day), :EVENTS => db.get_events(target_day) }
		db.close
		it
	end

	def receive_post(cgi)
		failed = false
		if(cgi.params['post'][0] == @post_value)
			target_time = Time.at(cgi.params['date'][0].to_i)
			target_day = target_time.day.to_s
			db = ScheduleDB.new(target_time.strftime(@db_name))
			# 更新の衝突チェック
			if(cgi.params['md5'][0] == db.get_md5(target_day))
				items = db.get_items(target_day)
				cgi.params['change_items'].each {|item|
					items.delete(item)							# いったん、削除
				}
				cgi.params['items'].each {|item|
					items[item] = true
				}
				cgi.params.each {|k, v|
					k =~ /^#point#(.+)/ and items[$1] = v[0].to_i
				}
				it = []; items.keys.sort.each {|key|
					it << (items[key] == true ? key : ("%s=%s" % [key, items[key]]))
				}
				db.set_mark(target_day, it.join(':') + ',' + cgi.params['events'][0].strip)
			else	# 衝突発生
				failed = :CONFLICT
			end
			db.close
		end
		total_points
		failed
	end

	def total_points
		points = {}
		(1..12).each {											# 1 年分前まで探しに行く
			File.exist?(@cur_month.strftime(@db_name)) or break
			each_day {|i|
				i[:ITEMS].each {|k, v|
					if(v != true)
						points[k] ||= 0
						points[k] += v.to_i
					end
				}
			}
			back_month
		}
		reset_month
		db = ScheduleDB.new(@cur_month.strftime(@db_name))
		points.each {|k, v|
			db.set_mark('#%s#' % k, v.to_s)
		}
		db.close
	end

	def get_point(item)
		reset_month
		db = ScheduleDB.new(@cur_month.strftime(@db_name))
		point = db.get_mark('#%s#' % item)
		db.close
		point
	end
end

__END__

